# Generated by Django 3.2.4 on 2021-08-22 13:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('magic_notifier', '0003_auto_20210819_1451'),
    ]

    operations = [
        migrations.AlterField(
            model_name='notification',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
    ]
